import { Injectable, Logger } from '@nestjs/common';
import { SchedulerRegistry } from '@nestjs/schedule';
import { CronJob } from 'cron';
import { SystemUsageService } from './system-usage.service';
import * as si from 'systeminformation';
import * as child_process from 'child_process';
import * as _ from 'loadsh';

@Injectable()
export class SystemUsageCron {
  private readonly logger = new Logger(SystemUsageCron.name);

  constructor(
    private readonly schedulerRegistry: SchedulerRegistry,
    private readonly systemUsageService: SystemUsageService,
  ) {}
  async handleGpuInfo(): Promise<{ usage: number }> {
    const { stdout, stderr } = await new Promise<{
      stdout: string;
      stderr: string;
    }>((res, rej) => {
      let stdout = '';
      let stderr = '';
      try {
        const execResult = child_process.execSync(
          process.env.GPU_MODEL === 'P800' ? 'xpu-smi -x -q' : 'nvidia-smi -x -q',
          { encoding: 'utf-8' }
        );
        stdout = execResult;
      } catch (error: any) {
        stderr = error.stderr ? error.stderr.toString() : error.message;
        throw error;
      }
      res({ stdout, stderr });
  });
    if (stderr) {
      this.logger.error('执行脚本时发生错误:', stderr);
      return { usage: 0 };
    }
    // this.logger.log('stdout', stdout)
    const reg = new RegExp(process.env.GPU_MODEL === 'P800' ? '<xpu_util>(\\d+)\\s*%</xpu_util>' : '<gpu_util>(\\d+)\\s*%</gpu_util>', 'g')
    const match = reg.exec(stdout)
    const usage = match ? match[1] : '0'
    return {usage: parseInt(usage)}
  }

  /**
   * 启动定时任务，用于定期记录系统使用情况
   */
  startCronJob() {
    // 每分钟执行一次
    const job = new CronJob('* * * * *', async () => {
      try {
        const [cpu, memory, gpuInfo] = await Promise.all([
          si.currentLoad(),
          si.mem(),
          this.handleGpuInfo(),
        ]);

        await this.systemUsageService.create({
          cpuUsage: parseFloat(cpu?.currentLoad.toFixed(2)),
          gpuUsage: parseFloat(gpuInfo.usage.toFixed(2)),
          memoryUsage: parseFloat((memory.active / memory.total*100).toFixed(2)),
        });

        this.logger.log(
          `系统指标已记录: CPU ${cpu.currentLoad.toFixed(2)}%, 内存 ${((memory.active / memory.total) * 100).toFixed(2)}% gpu ${gpuInfo.usage}`,
        );
      } catch (error: unknown) {
        if (error instanceof Error) {
          this.logger.error('记录系统指标失败', error.message);
        } else {
          this.logger.error('记录系统指标失败', String(error));
        }
      }
    });

    this.schedulerRegistry.addCronJob('systemUsageJob', job);
    job.start();
    this.logger.log('系统指标定时任务已启动');
  }
}
